#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['airline']

package_data = \
{'': ['*']}

extras_require = \
{'test': ['pytest ~=5.3', 'pytest-mock ~=2.0']}

setup(name='airline',
      version='0.5.1',
      description='Lightweight wide event logging to bring more observability to lambda functions.',
      author='namelessjon',
      author_email='jonathan.stott@gmail.com',
      url='https://github.com/namelessjon/airline',
      packages=packages,
      package_data=package_data,
      extras_require=extras_require,
     )
